@@ -2,12 +2,16 @@ from typing import Any, Callable, Dict, List, Optional, Union
 
 from itertools import dropwhile, takewhile
 from pathlib import Path
+from pprint import pformat
 import inspect
 import logging
 import os
+import re
 
 import wrapt
 
+from lunchbox.stopwatch import StopWatch
+
 LOG_LEVEL = os.environ.get('LOG_LEVEL', 'WARNING').upper()
 logging.basicConfig(level=LOG_LEVEL)
 LOGGER = logging.getLogger(__name__)
@@ -19,6 +23,27 @@ A library of miscellaneous tools.
 '''
 
 
+def to_snakecase(string):
+    # type: (str) -> str
+    '''
+    Converts a given string to snake_case.
+
+    Args:
+        string (str): String to be converted.
+
+    Returns:
+        str: snake_case string.
+    '''
+    output = re.sub('([A-Z]+)', r'_\1', string)
+    output = re.sub('-', '_', output)
+    output = re.sub(r'\.', '_', output)
+    output = re.sub(' ', '_', output)
+    output = re.sub('_+', '_', output)
+    output = re.sub('^_|_$', '', output)
+    output = output.lower()
+    return output
+
+
 def try_(function, item, return_item='item'):
     # type: (Callable[[Any], Any], Any, Any) -> Any
     '''
@@ -91,6 +116,62 @@ def relative_path(module, path):
     return output
 
 
+# LOGGING-----------------------------------------------------------------------
+def log_runtime(function, *args, message_=None, _testing=False, **kwargs):
+    # type (Callable, ..., Optional[str], bool, ...) -> Any
+    r'''
+    Logs the duration of given function called with given arguments.
+
+    Args:
+        function (function): Function to be called.
+        \*args (object, optional): Arguments.
+        message_ (str, optional): Message to be returned. Default: None.
+        _testing (bool, optional): Returns message if True. Default: False.
+        \*\*kwargs (object, optional): Keyword arguments.
+
+    Returns:
+        object: function(*args, **kwargs).
+    '''
+    # this may silently break file writes in multiprocessing
+    stopwatch = StopWatch()
+    stopwatch.start()
+    output = function(*args, **kwargs)
+    stopwatch.stop()
+
+    if message_ is not None:
+        message_ += f'\n         Runtime: {stopwatch.human_readable_delta}'
+    else:
+        message_ = f'''{function.__name__}
+         Runtime: {stopwatch.human_readable_delta}
+            Args: {pformat(args)}
+          Kwargs: {pformat(kwargs)}'''
+
+    if _testing:
+        return message_
+
+    LOGGER.info(message_)
+    return output
+
+
+@wrapt.decorator
+def runtime(wrapped, instance, args, kwargs):
+    # type: (Callable, Any, Any, Any) -> Any
+    r'''
+    Decorator for logging the duration of given function called with given
+    arguments.
+
+    Args:
+        wrapped (function): Function to be called.
+        instance (object): Needed by wrapt.
+        \*args (object, optional): Arguments.
+        \*\*kwargs (object, optional): Keyword arguments.
+
+    Returns:
+        function: Wrapped function.
+    '''
+    return log_runtime(wrapped, *args, **kwargs)
+
+
 # API---------------------------------------------------------------------------
 def get_function_signature(function):
     # type: (Callable) -> Dict
@@ -115,7 +196,7 @@ def get_function_signature(function):
 
 
 def api_function(wrapped=None, **kwargs):
-    # type: (Optional[Callable], **Any) -> Callable
+    # type: (Optional[Callable], Any) -> Callable
     '''
     A decorator that enforces keyword argument only function signatures and
     required keyword argument values when called.
